@@ -25,7 +25,7 @@ from packaging.utils import canonicalize_name
 from packaging.requirements import InvalidRequirement, Requirement
 from more_itertools import unique_everseen
 from ..utilities import lru_cache, run
-from ..errors import append_exception_text
+from ..errors import append_exception_text, get_exception_text
 
 _BUILTIN_DISTRIBUTION_NAMES: Tuple[str] = ("distribute",)
 # This variable tracks the absolute file paths from which a package has been
@@ -95,6 +95,8 @@ def get_installed_distributions() -> Dict[str, pkg_resources.Distribution]:
     """
     installed: Dict[str, pkg_resources.Distribution] = {}
     for distribution in pkg_resources.working_set:
+        if _distribution_is_editable(distribution):
+            _reload_distribution_information(distribution)
         installed[normalize_name(distribution.project_name)] = distribution
     return installed
 
@@ -247,24 +249,32 @@ def _distribution_is_editable(
 
 def _get_setup_py_distribution_name(path: str) -> str:
     current_directory: str = os.path.abspath(os.curdir)
+    directory: str = path
     try:
         if os.path.basename(path).lower() == "setup.py":
-            os.chdir(os.path.dirname(path))
+            directory = os.path.dirname(path)
+            os.chdir(directory)
         else:
             if not os.path.isdir(path):
-                path = os.path.dirname(path)
-            os.chdir(path)
-            path = os.path.join(path, "setup.py")
+                directory = os.path.dirname(path)
+            os.chdir(directory)
+            path = os.path.join(directory, "setup.py")
+        command: str = f"{quote(sys.executable)} {quote(path)} --name"
+        name: str
         try:
-            name: str = (
-                run(
-                    f"{quote(sys.executable)} {quote(path)} --name", echo=False
+            name = run(command, echo=False).strip().split("\n")[-1]
+        except Exception:
+            # re-write distribution and egg information and attempt to
+            # get the name again
+            setup_dist_egg_info(directory)
+            try:
+                run(command, echo=False).strip().split("\n")[-1]
+            except Exception:
+                warn(
+                    f"A package name could not be found in {directory}\n"
+                    f"Error ignored: {get_exception_text()}"
                 )
-                .strip()
-                .split("\n")[-1]
-            )
-        except OSError:
-            name = ""
+                name = ""
     finally:
         os.chdir(current_directory)
     return name
@@ -288,16 +298,19 @@ def get_setup_distribution_name(path: str) -> str:
     Get a distribution's name from setup.py or setup.cfg
     """
     return normalize_name(
-        _get_setup_py_distribution_name(path)
-        or _get_setup_cfg_distribution_name(path)
+        _get_setup_cfg_distribution_name(path)
+        or _get_setup_py_distribution_name(path)
     )
 
 
 def _setup(arguments: str) -> None:
-    run(
-        f"{quote(sys.executable)} setup.py {arguments}",
-        echo=False,
-    )
+    try:
+        run(
+            f"{quote(sys.executable)} setup.py {arguments}",
+            echo=False,
+        )
+    except Exception:
+        warn(f"Ignoring error: {get_exception_text()}")
 
 
 def _setup_location(location: str, arguments: Iterable[str]) -> None:
@@ -322,7 +335,31 @@ def setup_dist_egg_info(directory: str) -> None:
     directory = os.path.abspath(directory)
     if not os.path.isdir(directory):
         directory = os.path.dirname(directory)
-    return _setup_location(directory, ("-q dist_info", "-q egg_info"))
+    _setup_location(directory, ("-q dist_info", "-q egg_info"))
+
+
+def _reload_distribution_information(
+    distribution: pkg_resources.Distribution,
+) -> pkg_resources.Distribution:
+    if _distribution_is_editable(distribution):
+        setup_dist_egg_info(distribution.location)
+    # Clear the distribution information cache
+    for cached_property_name in (
+        "_pkg_info",
+        "_key",
+        "_parsed_version",
+        "_version",
+        "__dep_map",
+    ):
+        try:
+            delattr(distribution, cached_property_name)
+        except AttributeError:
+            pass
+    return distribution
+
+
+def reload_distribution_information(name: str) -> pkg_resources.Distribution:
+    return _reload_distribution_information(get_distribution(name))
 
 
 def setup_dist_info(directory: str) -> None:
@@ -364,7 +401,6 @@ def _get_location_distribution_name(location: str) -> str:
     Get a distribution name based on an installation location, or return
     an empty string if no distribution can be found
     """
-    setup_dist_egg_info(location)
     location = os.path.abspath(location)
 
     def _is_in_location(
@@ -441,6 +477,7 @@ def get_required_distribution_names(
     requirement_string: str,
     exclude: Iterable[str] = (),
     recursive: bool = True,
+    echo: bool = False,
 ) -> Set[str]:
     """
     Return a `set` of all distribution names which are required by the
@@ -455,6 +492,8 @@ def get_required_distribution_names(
       also halt recursive lookup of requirements for that distribution.
     - recursive (bool): If `True` (the default), required distributions will
       be obtained recursively.
+    - echo (bool) = False: If `True`, commands and responses executed in
+      subprocesses will be printed to `sys.stdout`
     """
     if isinstance(exclude, str):
         exclude = {normalize_name(exclude)}
@@ -465,6 +504,7 @@ def get_required_distribution_names(
             _get_pkg_requirement(requirement_string),
             exclude=exclude,
             recursive=recursive,
+            echo=echo,
         )
     )
 
@@ -488,7 +528,50 @@ def install_requirement(
     """
     if isinstance(requirement, str):
         requirement = Requirement(requirement)
-    return _install_requirement(requirement, echo)
+    return _install_requirement(requirement, echo=echo)
+
+
+def _install_requirement_string(
+    requirement_string: str,
+    name: str = "",
+    editable: bool = False,
+    echo: bool = False,
+) -> None:
+    uncaught_error: Optional[Exception] = None
+    flags: str
+    for flags in chain(
+        ("--user ", ""),
+        (
+            ("--user --force-reinstall ", "--force-reinstall ")
+            if editable
+            else ()
+        ),
+    ):
+        try:
+            run(
+                (
+                    f"{quote(sys.executable)} -m pip install {flags}"
+                    f"{quote(requirement_string)}"
+                ),
+                echo=echo,
+            )
+            uncaught_error = None
+            break
+        except OSError as error:
+            if (uncaught_error is None) or (not flags):
+                uncaught_error = error
+    if uncaught_error is not None:
+        append_exception_text(
+            uncaught_error,
+            (
+                f"\nCould not install {name}"
+                if name == requirement_string
+                else (f"\nCould not install {name} from {requirement_string}")
+            )
+            if name
+            else (f"\nCould not install {requirement_string}"),
+        )
+        raise uncaught_error
 
 
 def _install_requirement(
@@ -512,47 +595,30 @@ def _install_requirement(
     # If the requirement is installed and editable, re-install from
     # the editable location
     if distribution and editable:
-        # Refresh metadata
-        setup_dist_egg_info(distribution.location)
         # Assemble a requirement specifier for the editable install
         requirement_string = distribution.location
         if requirement.extras:
             requirement_string = (
                 f"{requirement_string}[{','.join(requirement.extras)}]"
             )
-    # First attempt a user install, then a system install
-    try:
-        run(
-            (
-                f"{quote(sys.executable)} -m pip install --user "
-                f"{quote(requirement_string)}"
-            ),
-            echo=echo,
-        )
-    except OSError:
-        try:
-            # If a user installation failed, attempt a system installation
-            run(
-                (
-                    f"{quote(sys.executable)} -m pip install "
-                    f"{quote(requirement_string)}"
-                ),
-                echo=echo,
-            )
-        except OSError as error:
-            append_exception_text(
-                error, f"Could not install {name} from {requirement_string}"
-            )
-            raise error
+    _install_requirement_string(
+        requirement_string=requirement_string,
+        name=name,
+        editable=editable,
+        echo=echo,
+    )
     # Refresh the metadata
-    if distribution and editable:
-        setup_dist_egg_info(distribution.location)
+    if distribution:
+        _reload_distribution_information(distribution)
     else:
         refresh_working_set()
 
 
 def _get_pkg_requirement_distribution(
-    requirement: pkg_resources.Requirement, name: str, reinstall: bool = True
+    requirement: pkg_resources.Requirement,
+    name: str,
+    reinstall: bool = True,
+    echo: bool = False,
 ) -> Optional[pkg_resources.Distribution]:
     if name in _BUILTIN_DISTRIBUTION_NAMES:
         return None
@@ -561,15 +627,15 @@ def _get_pkg_requirement_distribution(
     except KeyError:
         if not reinstall:
             raise
-        warn(
-            f'The required distribution "{name}" was not installed, '
-            "attempting to install it now..."
-        )
+        if echo:
+            warn(
+                f'The required distribution "{name}" was not installed, '
+                "attempting to install it now..."
+            )
         # Attempt to install the requirement...
-        install_requirement(requirement)
-        refresh_working_set()
+        install_requirement(requirement, echo=echo)
         return _get_pkg_requirement_distribution(
-            requirement, name, reinstall=False
+            requirement, name, reinstall=False, echo=echo
         )
 
 
@@ -577,6 +643,7 @@ def _iter_requirement_names(
     requirement: pkg_resources.Requirement,
     exclude: Set[str],
     recursive: bool = True,
+    echo: bool = False,
 ) -> Iterable[str]:
     name: str = normalize_name(requirement.project_name)
     extras: Set[str] = set(map(normalize_name, requirement.extras))
@@ -587,12 +654,9 @@ def _iter_requirement_names(
     exclude.add(name)
     distribution: Optional[
         pkg_resources.Distribution
-    ] = _get_pkg_requirement_distribution(requirement, name)
+    ] = _get_pkg_requirement_distribution(requirement, name, echo=echo)
     if distribution is None:
         return ()
-    # Ensure requirements are up-to-date
-    if _distribution_is_editable(distribution):
-        setup_dist_egg_info(distribution.location)
     requirements: List[pkg_resources.Requirement] = distribution.requires(
         extras=tuple(sorted(extras))
     )
@@ -608,6 +672,7 @@ def _iter_requirement_names(
                 | (lateral_exclude - {_get_pkg_requirement_name(requirement_)})
             ),
             recursive=recursive,
+            echo=echo,
         )
 
     def not_excluded(name: str) -> bool:
@@ -627,6 +692,7 @@ def _iter_requirement_names(
 
 def _iter_requirement_strings_required_distribution_names(
     requirement_strings: Iterable[str],
+    echo: bool = False,
 ) -> Iterable[str]:
     visited_requirement_strings: Set[str] = set()
     if isinstance(requirement_strings, str):
@@ -639,9 +705,9 @@ def _iter_requirement_strings_required_distribution_names(
                     requirement_string
                 )
                 visited_requirement_strings.add(requirement_string)
-                return get_required_distribution_names(requirement_string) | {
-                    name
-                }
+                return get_required_distribution_names(
+                    requirement_string, echo=echo
+                ) | {name}
             except KeyError:
                 pass
         return set()
@@ -653,6 +719,7 @@ def _iter_requirement_strings_required_distribution_names(
 
 def get_requirements_required_distribution_names(
     requirements: Iterable[str] = (),
+    echo: bool = False,
 ) -> Set[str]:
     """
     Get the distributions required by one or more specified distributions or
@@ -686,6 +753,7 @@ def get_requirements_required_distribution_names(
                         ),
                     )
                 ),
+                echo=echo,
             ),
             key=lambda name: name.lower(),
         )
